"use strict";
/**
 * @file Governance
 * @desc These methods facilitate interactions with the Governor smart contract.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.castVoteWithReason = exports.createVoteSignature = exports.castVoteBySig = exports.castVote = void 0;
const eth = __importStar(require("./eth"));
const helpers_1 = require("./helpers");
const constants_1 = require("./constants");
const EIP712_1 = require("./EIP712");
/**
 * Submit a vote on a Compound Governance proposal.
 *
 * @param {string} proposalId The ID of the proposal to vote on. This is an
 *     auto-incrementing integer in the Governor contract.
 * @param {number} support A number value of 0, 1, or 2 for the proposal vote.
 *     The numbers correspond to 'in-favor', 'against', and 'abstain'
 *     respectively.
 * @param {CallOptions} [options] Options to set for a transaction and Ethers.js
 *     method overrides.
 *
 * @returns {object} Returns an Ethers.js transaction object of the vote
 *     transaction.
 *
 * @example
 *
 * ```
 * const compound = new Compound(window.ethereum);
 *
 * (async function() {
 *   const castVoteTx = await compound.castVote(12, 1);
 *   console.log('Ethers.js transaction object', castVoteTx);
 * })().catch(console.error);
 * ```
 */
async function castVote(proposalId, support, options = {}) {
    await (0, helpers_1.netId)(this);
    const errorPrefix = 'Compound [castVote] | ';
    if (typeof proposalId !== 'number') {
        throw Error(errorPrefix + 'Argument `proposalId` must be an integer.');
    }
    if (typeof support !== 'number') {
        throw Error(errorPrefix + 'Argument `support` must be an integer (0, 1, or 2).');
    }
    const governorAddress = (0, constants_1.getAddress)(this._network.name, 'GovernorBravo');
    const trxOptions = options;
    trxOptions._compoundProvider = this._provider;
    trxOptions.abi = constants_1.abi.GovernorBravo;
    const parameters = [proposalId, support];
    const method = 'castVote';
    return eth.trx(governorAddress, method, parameters, trxOptions);
}
exports.castVote = castVote;
/**
 * Submit a vote on a Compound Governance proposal using an EIP-712 signature.
 *
 * @param {string} proposalId The ID of the proposal to vote on. This is an
 *     auto-incrementing integer in the Governor contract.
 * @param {number} support A number value of 0, 1, or 2 for the proposal vote.
 *     The numbers correspond to 'in-favor', 'against', and 'abstain'
 *     respectively.
 * @param {object} signature An object that contains the v, r, and, s values of
 *     an EIP-712 signature.
 * @param {CallOptions} [options] Options to set for a transaction and Ethers.js
 *     method overrides.
 *
 * @returns {object} Returns an Ethers.js transaction object of the vote
 *     transaction.
 *
 * @example
 * ```
 * const compound = new Compound(window.ethereum);
 *
 * (async function() {
 *   const castVoteTx = await compound.castVoteBySig(
 *     12,
 *     1,
 *     {
 *       v: '0x1b',
 *       r: '0x130dbcd2faca07424c033b4479687cc1deeb65f08509e3ab397988cc4c6f2e78',
 *       s: '0x1debcb8250262f23906b1177161f0c7c9aa3641e6bff5b6f5c88a6bb78d5d8cd'
 *     }
 *   );
 *   console.log('Ethers.js transaction object', castVoteTx);
 * })().catch(console.error);
 * ```
 */
async function castVoteBySig(proposalId, support, signature, options = {}) {
    await (0, helpers_1.netId)(this);
    const errorPrefix = 'Compound [castVoteBySig] | ';
    if (typeof proposalId !== 'number') {
        throw Error(errorPrefix + 'Argument `proposalId` must be an integer.');
    }
    if (typeof support !== 'number') {
        throw Error(errorPrefix + 'Argument `support` must be an integer (0, 1, or 2).');
    }
    if (!Object.isExtensible(signature) || !signature.v || !signature.r || !signature.s) {
        throw Error(errorPrefix +
            'Argument `signature` must be an object that ' +
            'contains the v, r, and s pieces of an EIP-712 signature.');
    }
    const governorAddress = (0, constants_1.getAddress)(this._network.name, 'GovernorBravo');
    const trxOptions = options;
    trxOptions._compoundProvider = this._provider;
    trxOptions.abi = constants_1.abi.GovernorBravo;
    const { v, r, s } = signature;
    const parameters = [proposalId, support, v, r, s];
    const method = 'castVoteBySig';
    return eth.trx(governorAddress, method, parameters, trxOptions);
}
exports.castVoteBySig = castVoteBySig;
/**
 * Create a vote signature for a Compound Governance proposal using EIP-712.
 *     This can be used to create an 'empty ballot' without burning gas. The
 *     signature can then be sent to someone else to post to the blockchain.
 *     The recipient can post one signature using the `castVoteBySig` method.
 *
 * @param {string} proposalId The ID of the proposal to vote on. This is an
 *     auto-incrementing integer in the Governor contract.
 * @param {number} support A number value of 0, 1, or 2 for the proposal vote.
 *     The numbers correspond to 'in-favor', 'against', and 'abstain'
 *     respectively. To create an 'empty ballot' call this method thrice using
 *     `0`, `1`, and then `2` for this parameter.
 *
 * @returns {object} Returns an object that contains the `v`, `r`, and `s`
 *     components of an Ethereum signature as hexadecimal strings.
 *
 * @example
 * ```
 * const compound = new Compound(window.ethereum);
 *
 * (async () => {
 *
 *   const voteForSignature = await compound.createVoteSignature(20, 1);
 *   console.log('voteForSignature', voteForSignature);
 *
 *   const voteAgainstSignature = await compound.createVoteSignature(20, 0);
 *   console.log('voteAgainstSignature', voteAgainstSignature);
 *
 * })().catch(console.error);
 * ```
 */
async function createVoteSignature(proposalId, support) {
    await (0, helpers_1.netId)(this);
    const provider = this._provider;
    const governorAddress = (0, constants_1.getAddress)(this._network.name, 'GovernorBravo');
    const chainId = this._network.id;
    const domain = {
        name: 'Compound Governor Bravo',
        chainId,
        verifyingContract: governorAddress,
    };
    const primaryType = 'Ballot';
    const message = { proposalId, support };
    const types = {
        EIP712Domain: [
            { name: 'name', type: 'string' },
            { name: 'chainId', type: 'uint256' },
            { name: 'verifyingContract', type: 'address' },
        ],
        Ballot: [
            { name: 'proposalId', type: 'uint256' },
            { name: 'support', type: 'uint8' },
        ],
    };
    const signer = provider.getSigner ? provider.getSigner() : provider;
    const signature = await (0, EIP712_1.sign)(domain, primaryType, message, types, signer);
    return signature;
}
exports.createVoteSignature = createVoteSignature;
/**
 * Submit a Compound Governance proposal vote with a reason.
 *
 * @param {string} proposalId The ID of the proposal to vote on. This is an
 *     auto-incrementing integer in the Governor contract.
 * @param {number} support A number value of 0, 1, or 2 for the proposal vote.
 *     The numbers correspond to 'in-favor', 'against', and 'abstain'
 *     respectively.
 * @param {string} reason A string of the reason for a vote selection.
 * @param {CallOptions} [options] Options to set for a transaction and Ethers.js
 *     method overrides.
 *
 * @returns {object} Returns an Ethers.js transaction object of the vote
 *     transaction.
 *
 * @example
 *
 * ```
 * const compound = new Compound(window.ethereum);
 *
 * (async function() {
 *   const castVoteTx = await compound.castVoteWithReason(12, 1, 'I vote YES because...');
 *   console.log('Ethers.js transaction object', castVoteTx);
 * })().catch(console.error);
 * ```
 */
async function castVoteWithReason(proposalId, support, reason, options = {}) {
    await (0, helpers_1.netId)(this);
    const errorPrefix = 'Compound [castVoteWithReason] | ';
    if (typeof proposalId !== 'number') {
        throw Error(errorPrefix + 'Argument `proposalId` must be an integer.');
    }
    if (typeof support !== 'number') {
        throw Error(errorPrefix + 'Argument `support` must be an integer (0, 1, or 2).');
    }
    if (typeof reason !== 'string') {
        throw Error(errorPrefix + 'Argument `reason` must be a string.');
    }
    const governorAddress = (0, constants_1.getAddress)(this._network.name, 'GovernorBravo');
    const trxOptions = options;
    trxOptions._compoundProvider = this._provider;
    trxOptions.abi = constants_1.abi.GovernorBravo;
    const parameters = [proposalId, support, reason];
    const method = 'castVoteWithReason';
    return eth.trx(governorAddress, method, parameters, trxOptions);
}
exports.castVoteWithReason = castVoteWithReason;
//# sourceMappingURL=gov.js.map